#ifndef __KONECONFIG_SETTINGS_PAGE_H__
#define __KONECONFIG_SETTINGS_PAGE_H__

/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/kone.h"
#include <gtk/gtk.h>

G_BEGIN_DECLS

#define KONECONFIG_SETTINGS_PAGE_TYPE (koneconfig_settings_page_get_type())
#define KONECONFIG_SETTINGS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), KONECONFIG_SETTINGS_PAGE_TYPE, KoneconfigSettingsPage))
#define IS_KONECONFIG_SETTINGS_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), KONECONFIG_SETTINGS_PAGE_TYPE))

typedef struct _KoneconfigSettingsPage KoneconfigSettingsPage;

GType koneconfig_settings_page_get_type(void);
GtkWidget *koneconfig_settings_page_new(void);

guint koneconfig_settings_page_get_actual_profile(KoneconfigSettingsPage *settings_page);
void koneconfig_settings_page_set_actual_profile(KoneconfigSettingsPage *settings_page, guint profile_number);

void koneconfig_settings_page_clear_info(KoneconfigSettingsPage *settings_page);
void koneconfig_settings_page_set_info(KoneconfigSettingsPage *settings_page, guint firmware_version);

void koneconfig_settings_page_set_weight(KoneconfigSettingsPage *settings_page, guint weight);
void koneconfig_settings_page_clear_weight(KoneconfigSettingsPage *settings_page);

G_END_DECLS

#endif
